package brookes.joelmarty.semantic.bookclub.webapp.model;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import lombok.Data;
import lombok.EqualsAndHashCode;
import brookes.joelmarty.semantic.bookclub.webapp.utils.ModelException;

import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.StmtIterator;
import com.hp.hpl.jena.sparql.vocabulary.FOAF;

@Data
@EqualsAndHashCode(callSuper=true)
public class Member extends AbstractModel {
	
	/**
	 * email adress for the member
	 */
	private String email;
	
	/**
	 * Full name of member
	 */
	private String name;
	
	/**
	 * list of books owned by member
	 */
	private List<Book> books;
	
	/**
	 * list of known persons by this member
	 */
	private List<Member> acquaintances;

	/**
	 * Load a member from a resource.
	 * @param resource The resource representing a member.
	 * @param model The Model instance to load the resource from.
	 * @param deep If true, will also load this member's books and acquaintances.
	 * @return The member the resource represents
	 * @throws ModelException
	 */
	public static Member fromResource(Resource resource, Model model, boolean deep) throws ModelException {
		Member member = new Member();
		
		try {
			// set id
			// there seems to be a bug in Resource.getLocalName()
			// I have to strip the id from the URI instead
			String memberUri = resource.getURI();
			String[] uriParts = memberUri.split("/");
			String maybeMemberId = uriParts[uriParts.length - 1];
			String[] hashedUriParts = maybeMemberId.split("#");
			String memberId = hashedUriParts[hashedUriParts.length - 1];
			
			member.setId(UUID.fromString(memberId));
			
			// set email
			Statement emailStatement = resource.getProperty(FOAF.mbox);
			if (emailStatement != null)
				member.setEmail(emailStatement.getLiteral().getString());
			else
				member.setEmail("");
			
			// set name
			Statement nameStatement = resource.getProperty(FOAF.name);
			if (nameStatement != null)
				member.setName(nameStatement.getLiteral().getString());
			else
				member.setName("");
			
			if (deep) {
				// set books
				List<Book> ownedBooks = new ArrayList<Book>();
				StmtIterator bookIter = resource.listProperties(model
						.createProperty(ModelManager.SCHEMA_URI, "ownBook"));
				while (bookIter.hasNext()) {
					Resource bookResource = bookIter.next().getObject()
							.asResource();
					Book ownedBook = Book.fromResource(bookResource, model);
					ownedBooks.add(ownedBook);
				}
				member.setBooks(ownedBooks);
				// set acquaintances
				List<Member> acquaintances = new ArrayList<Member>();
				StmtIterator acqIter = resource.listProperties(FOAF.knows);
				while (acqIter.hasNext()) {
					Resource knownMemberResource = acqIter.next().getObject().asResource();
					Member knownMember = Member.fromResource(knownMemberResource, model, false);
					acquaintances.add(knownMember);
				}
				member.setAcquaintances(acquaintances);
			} else {
				member.setBooks(new ArrayList<Book>());
				member.setAcquaintances(new ArrayList<Member>());
			}
			
		} catch (Exception e) {
			throw new ModelException("Error while fetching member #" + member.getId(), e);
		}
		
		return member;
	}

}
