package brookes.joelmarty.semantic.bookclub.webapp.model;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Scope;

import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.tdb.TDBFactory;
import com.hp.hpl.jena.util.FileManager;

/**
 * ModelManager class that offers configuration facilities to the RDF model.<p>
 * Annotated with '@Configuration' that tells spring that this class offers some beans that may be injected somewhere else.
 * @author joel
 *
 */
@Configuration
public class ModelManager {
	
	/**
	 * The model file name. Hardcoded but should be provided in a property file.
	 */
	public static final String MODEL_FILE = "data/bookclub.rdf";
	
	/**
	 * The schema file name.
	 */
	public static final String SCHEMA_FILE = "data/bookclub.rdfs";
	
	/**
	 * The name of the model.
	 */
	public static final String MODEL_NAME = "bookclub";
	
	/**
	 * The URI to the model
	 */
	public static final String MODEL_URI = "http://cms.brookes.ac.uk/modules/P00775/2011/10093745/";
	
	/**
	 * The RDFS schema uri
	 */
	public static final String SCHEMA_URI = "http://cms.brookes.ac.uk/modules/P00775/2011/10093745/schema#";

	/**
	 * String representing the name of the bean (a String actually) holding the book concept URI
	 */
	public static final String CONCEPT_BOOK = "book_concept";
	
	/**
	 * String representing the name of the bean (a String actually) holding the member concept URI
	 */
	public static final String CONCEPT_MEMBER = "member_concept";
	
	/**
	 * String representing the name of the bean (a String actually) holding the lending concept URI
	 */
	public static final String CONCEPT_LENDING = "lending_concept";
	
	
	private static final Logger LOG = LoggerFactory.getLogger(Book.class);
	
	/**
	 * The configuration method that is used to create a Model instance as a singleton.
	 * @return a Model instance pointing to the model file.
	 */
	@Bean(name="model")
	@Scope("singleton")
	public Model getClubModel() {
		
		String directory = "data/model/";
		
		Model model = null;
		Model schema = null;
		try {
			Model tdbmodel = TDBFactory.createModel(directory);
			
			schema = FileManager.get().loadModel(SCHEMA_FILE);
			model = ModelFactory.createRDFSModel(schema, tdbmodel);
		} catch (Exception e) {
			LOG.error("Error while opening model at " + MODEL_FILE,e);
		}
		return model;
	}
	
	/**
	 * The configuration method returning the URI to the book concept.
	 * @return String representing the URI for books
	 */
	@Bean(name=CONCEPT_BOOK)
	public String getBookConcept() {
		return SCHEMA_URI + "book";
	}
	
	/**
	 * The configuration method returning the URI to the member concept.
	 * @return String representing the URI for members
	 */
	@Bean(name=CONCEPT_MEMBER)
	public String getMemberConcept() {
		return SCHEMA_URI + "member";
	}
	
	/**
	 * The configuration method returning the URI to the lending concept.
	 * @return String representing the URI for lendings
	 */
	@Bean(name=CONCEPT_LENDING)
	public String getLendingConcept() {
		return SCHEMA_URI + "lending";
	}
}
