package brookes.joelmarty.semantic.bookclub.webapp.service;

import java.net.URI;
import java.util.ArrayList;
import java.util.List;

import javax.ws.rs.Consumes;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.MultivaluedMap;
import javax.ws.rs.core.Response;

import org.springframework.beans.BeansException;
import org.springframework.stereotype.Service;

import brookes.joelmarty.semantic.bookclub.webapp.model.Book;
import brookes.joelmarty.semantic.bookclub.webapp.model.Member;
import brookes.joelmarty.semantic.bookclub.webapp.model.ModelManager;
import brookes.joelmarty.semantic.bookclub.webapp.utils.ModelException;

import com.hp.hpl.jena.query.QueryExecution;
import com.hp.hpl.jena.query.QueryExecutionFactory;
import com.hp.hpl.jena.query.QuerySolution;
import com.hp.hpl.jena.query.QuerySolutionMap;
import com.hp.hpl.jena.query.ResultSet;
import com.hp.hpl.jena.rdf.model.Literal;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.vocabulary.DCTerms;

@Service
@Path("books")
public class BookService extends AbstractModelService {

	/**
	 * Method to add a new book to the model.<p>
	 * @param subject The book to add
	 * @throws ModelException exception raised whenever an error occurs
	 */
	@POST
	@Path("add")
	@Consumes(MediaType.APPLICATION_FORM_URLENCODED)
	public Response add(MultivaluedMap<String, String> form) {

		Book b = new Book();
		
		try {
		
			Resource bookClassResource = model.getResource(bookModelUri);
			
			Resource r = model.createResource(b.getUri(), bookClassResource);
			
			// adding isbn property
			Property isbnProperty = model.createProperty(ModelManager.SCHEMA_URI, "hasIsbn");
			r.addLiteral(isbnProperty, model.createLiteral(form.getFirst("isbn")));
			
			// adding title
			r.addLiteral(DCTerms.title, form.getFirst("title"));
			
			// author
			r.addLiteral(DCTerms.creator, form.getFirst("author"));

			// publisher
			r.addLiteral(DCTerms.publisher, form.getFirst("publisher"));
			
			// subject
			r.addLiteral(DCTerms.subject, model.createLiteral(form.getFirst("subject")));
			
			// year of publication
			Literal l = model.createTypedLiteral(Integer.parseInt(form.getFirst("year")));
			r.addLiteral(DCTerms.issued, l);

			model.commit();
			
		} catch (NumberFormatException e) {
			LOG.error("Error while creating book #" + b.getId().toString() + ": entered date is invalid");
			try {
				deleteMdl(b.getId().toString());
			} catch (ModelException me) {
				throw new WebApplicationException(Response.serverError().entity("failed in an attempt to restore model").build());
			}
			throw new WebApplicationException(Response.serverError().entity("publication date is invalid!").build());
		} catch (Exception e) {
			LOG.error("Error while creating book #" + b.getId().toString(), e);
			try {
				deleteMdl(b.getId().toString());
			} catch (ModelException me) {
				throw new WebApplicationException(Response.serverError().entity("failed in an attempt to restore model").build());
			}
		} 
		
		return Response.created(URI.create("/books")).build();
	}
	
	/**
	 * Method to edit a book in the model.
	 * @throws ModelException Raised whenever an error occurs.
	 */
	@POST
	@Consumes("application/x-www-form-urlencoded")
	@Path("edit/{id}")
	public Response edit(@PathParam("id") String id, MultivaluedMap<String, String> formValues) {
		try {
			Resource r = get(id);
			
			// updating title
			r.removeAll(DCTerms.title);
			r.addLiteral(DCTerms.title, formValues.getFirst("title"));
			
			// author
			r.removeAll(DCTerms.creator);
			r.addLiteral(DCTerms.creator, formValues.getFirst("author"));
			
			// publisher
			r.removeAll(DCTerms.publisher);
			r.addProperty(DCTerms.publisher, formValues.getFirst("publisher"));
			
			// subject
			r.removeAll(DCTerms.subject);
			r.addLiteral(DCTerms.subject, model.createLiteral(formValues.getFirst("subject")));
			
			// year of publication
			r.removeAll(DCTerms.issued);
			Literal l = model.createTypedLiteral(Integer.parseInt(formValues.getFirst("year")));
			r.addLiteral(DCTerms.issued, l);
			
			model.commit();
		} catch (Exception e) {
			LOG.error("error while adding book #" + id);
			throw new WebApplicationException(Response.serverError().entity(e.getLocalizedMessage()).build());
		}
		
		return Response.ok().build();
	}
	
	/**
	 * Method to delete a book from the model.
	 * @throws ModelException Raised whenever an error occurs.
	 */
	@GET
	@Path("delete/{id}")
	public Response delete(@PathParam("id") String id) {
		
		try {
			deleteMdl(id);
		} catch (ModelException e) {
			return Response.serverError().build();
		}
		
		return Response.ok().build();
	}
	
	@GET
	@Produces("text/html")
	public Response list() throws ModelException {
		List<Book> books = new ArrayList<Book>();
		try {
			List<Resource> resources = AbstractModelService.list(bookModelUri, model);
			
			for (Resource r : resources) {
				books.add(Book.fromResource(r, model));
			}
		} catch (BeansException e) {
			LOG.error("Spring error while fetching books");
			throw new ModelException(e);
		} catch (Exception e) {
			LOG.error("Error while fetching books");
			throw new ModelException(e);
		}
		
		return Response.ok(fm.render("books.ftl", "books", books)).build();
	}

	@GET
	@Path("{id}")
	@Produces("text/html")
	public Response show(@PathParam("id") String id) {
		
		Book b = null;
		List<Member> owners = new ArrayList<Member>();
		
		String queryString =
			"PREFIX bc: <http://cms.brookes.ac.uk/modules/P00775/2011/10093745/schema#> " +
			"SELECT ?member " +
			"WHERE { ?member bc:ownBook ?book }";
		
		try {
			Resource r = get(id);
			
			if(r != null) {
				b = Book.fromResource(r, model);
				
				QuerySolutionMap varMapping = new QuerySolutionMap();
				varMapping.add("book", r);
				
				QueryExecution query = QueryExecutionFactory.create(queryString, model, varMapping);
				ResultSet res = query.execSelect();
				
				while(res.hasNext()) {
					
					QuerySolution soln = res.nextSolution();
					Resource foundResource = soln.getResource("member");
					owners.add(Member.fromResource(foundResource, model, false));
				}
				
			} else {
				throw new ModelException("Book #" + id + " doesn't exist!");
			}
		} catch (Exception e) {
			LOG.error("Error while fetching book #" + id);
			return Response.serverError().build();
		}
		return Response.ok(fm.render("book.ftl", "book", b, "owners", owners)).build();
	}
	
	@GET
	@Path("add")
	public Response showAdd() {
		
		return Response.ok(fm.render("bookform.ftl", "operation", "add")).build();
	}
	
	@GET
	@Path("edit/{id}")
	public Response showEdit(@PathParam("id") String id) {
		
		Book b = null;
		try {
			Resource bookResource = get(id);
			b = Book.fromResource(bookResource, model);
		} catch (Exception e) {
			return Response.serverError().build();
		}
		
		return Response.ok(fm.render("bookform.ftl", "operation", "edit", "book", b)).build();
	}
	
}
