package brookes.joelmarty.semantic.bookclub.webapp.service;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.ws.rs.Consumes;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.MultivaluedMap;
import javax.ws.rs.core.Response;

import org.springframework.stereotype.Service;

import brookes.joelmarty.semantic.bookclub.webapp.model.Book;
import brookes.joelmarty.semantic.bookclub.webapp.model.Lending;
import brookes.joelmarty.semantic.bookclub.webapp.model.Member;
import brookes.joelmarty.semantic.bookclub.webapp.model.ModelManager;
import brookes.joelmarty.semantic.bookclub.webapp.utils.ModelException;

import com.hp.hpl.jena.query.QueryExecution;
import com.hp.hpl.jena.query.QueryExecutionFactory;
import com.hp.hpl.jena.query.QuerySolution;
import com.hp.hpl.jena.query.QuerySolutionMap;
import com.hp.hpl.jena.query.ResultSet;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.Resource;

@Service
@Path("/lendings")
public class LendingService extends AbstractModelService {

	/**
	 * List all lendings
	 * @return Response object representing the web page.
	 */
	@GET
	@Produces("text/html")
	public Response list() {
		
		List<Lending> lendings = new ArrayList<Lending>();
		
		try {
			List<Resource> lendingsResources = AbstractModelService.list(lendingModelUri, model);
			
			for (Resource resource : lendingsResources) {
				Lending l = Lending.fromResource(resource, model);
				lendings.add(l);
			}
		
		} catch (Exception e) {
			LOG.error("error while fetching lendings");
			throw new WebApplicationException(Response.serverError().entity(e.getLocalizedMessage()).build());
		}
		
		return Response.ok(fm.render("lendings.ftl", "lendings", lendings)).build();
	}
	
	/**
	 * Deletes a lending from the store.
	 * @param id the uuid of the lending.
	 * @return
	 */
	@GET
	@Path("delete/{id}")
	public Response delete(@PathParam("id") String id) {
		try {
			deleteMdl(id);
		} catch (ModelException e) {
			LOG.error("error while deleting lending #" + id);
			throw new WebApplicationException(Response.serverError().entity(e.getLocalizedMessage()).build());
		}
		
		return Response.ok().build();
	}
	
	/**
	 * Effectively adds a loan to the store
	 * @param form the form object containing the information
	 * @return
	 */
	@POST
	@Path("add")
	@Consumes("application/x-www-form-urlencoded")
	public Response add(MultivaluedMap<String, String> form) {
		
		Lending l = new Lending();
		
		try {
		
		Resource memberResource = model.getResource(lendingModelUri);
		
		Resource r = model.createResource(l.getUri(), memberResource);

		String[] bookCompositeValue = form.getFirst("book").split("&");
		String bookId = bookCompositeValue[0];
		String lenderId = bookCompositeValue[1];
		
		// adding the borrowed book property
		Property borrowedBookProperty = model.createProperty(ModelManager.SCHEMA_URI, "borrowedBook");
		r.addProperty(borrowedBookProperty, bookId);
		
		// adding the lender property
		Property borrowedFromProperty = model.createProperty(ModelManager.SCHEMA_URI, "borrowedFrom");
		r.addProperty(borrowedFromProperty, lenderId);
		
		// adding the borrower property
		Property lendToProperty = model.createProperty(ModelManager.SCHEMA_URI, "lendTo");
		r.addProperty(lendToProperty, form.getFirst("borrower"));
		
		// adding the borrowed date property
		Property borrowedOnProperty = model.createProperty(ModelManager.SCHEMA_URI, "borrowedOn");
		r.addLiteral(borrowedOnProperty, model.createTypedLiteral(Calendar.getInstance()));
			
		} catch (Exception e) {
			LOG.error("error while adding lending #" + l.getId().toString());
			try {
				deleteMdl(l.getId().toString());
			} catch (ModelException me) {
				throw new WebApplicationException(Response.serverError().entity("failed in an attempt to restore model").build());
			}
			throw new WebApplicationException(Response.serverError().entity(e.getLocalizedMessage()).build());
		}
	
		return Response.ok().build();
	}
	
	/**
	 * Displays the new loan form
	 * @param memberId the id of the lender
	 * @return
	 */
	@GET
	@Path("add/{id}")
	@Produces("text/html")
	public Response showAdd(@PathParam("id") String memberId) {
		
		Member borrowingMember = null;
		
		Map<String, String> mapBooksToMembers = new HashMap<String, String>();
		
		String queryString =
			"PREFIX dct: <http://purl.org/dc/terms/> \n" +
			"PREFIX foaf: <http://xmlns.com/foaf/0.1/> \n" +
			"PREFIX bc: <http://cms.brookes.ac.uk/modules/P00775/2011/10093745/schema#> \n" +
			"SELECT DISTINCT ?book ?p1 ?p2 WHERE { " +
			"{ ?member foaf:knows ?p1 . " +
			"{ ?p1 bc:ownBook ?book } " +
			"UNION { ?p1 foaf:knows ?p2 . " +
			"        ?p2 bc:ownBook ?book } " +
			"} " +
			"MINUS { " +
			"?member bc:ownBook ?book " +
			"}}";
		
		try {
			Resource borrowingMemberResource = get(memberId);
			borrowingMember = Member.fromResource(borrowingMemberResource, model, false);
			
			
			QuerySolutionMap varMapping = new QuerySolutionMap();
			varMapping.add("member", borrowingMemberResource);
			
			QueryExecution query = QueryExecutionFactory.create(queryString, model, varMapping);
			ResultSet res = query.execSelect();
			
			while(res.hasNext()) {
				QuerySolution soln = res.nextSolution();
				Resource foundBookResource = soln.getResource("book");
				Resource firstLevelBookOwnerResource = soln.getResource("p1");
				Resource secondLevelBookOwnerResource = soln.getResource("p2");
				
				Resource bookOwnerResource = null;
				// if the book comes from a first level acquaintance
				if (firstLevelBookOwnerResource != null) {
					bookOwnerResource = firstLevelBookOwnerResource;
				} else if (secondLevelBookOwnerResource != null) {
					// the book must then come from a second level acquaintance
					bookOwnerResource = secondLevelBookOwnerResource;
				} else {
					throw new WebApplicationException();
				}
				
				Member m = Member.fromResource(bookOwnerResource, model, false);
				Book b = Book.fromResource(foundBookResource, model);
				
				if(m.getId().toString() != memberId)
					mapBooksToMembers.put(b.getId().toString() + "&" + m.getId().toString(),
										  b.getTitle() + " from " + m.getName());
			}
		} catch (Exception e) {
			LOG.error("Error while fetching lendable books");
			throw new WebApplicationException(Response.serverError().entity(e.getLocalizedMessage()).build());
		}
		
		return Response.ok(fm.render("lendingform.ftl",
									 "borrower", borrowingMember,
									 "bookstomembers",mapBooksToMembers)).build();
	}
	
	/**
	 * Displays expired lendings
	 * @return
	 */
	@GET
	@Path("expired")
	@Produces("text/html")
	public Response showExpired() {
		
		String dateFormat = "yyyy-MM-dd";
		SimpleDateFormat sdf = new SimpleDateFormat(dateFormat);
		
		Calendar calendar = Calendar.getInstance();
		calendar.add(Calendar.MONTH, -1);
		
		String formattedDate = "\"" + sdf.format(calendar.getTime()) + "\"^^xsd:date";
		
		
		String queryString =
			"PREFIX bc: <http://cms.brookes.ac.uk/modules/P00775/2011/10093745/schema#> " +
			"PREFIX xsd: <http://www.w3.org/2001/XMLSchema#> " +
			"SELECT ?lending " +
			"WHERE { " +
			"?lending bc:borrowedOn ?date . " +
			"FILTER (?date <= " + formattedDate + ") " +
			"}";

		List<Lending> expiredLendings = new ArrayList<Lending>();
		
		try {

			QueryExecution query = QueryExecutionFactory.create(queryString, model);
			ResultSet res = query.execSelect();
			
			while(res.hasNext()) {
				QuerySolution soln = res.nextSolution();
				Resource lendingResource = soln.getResource("lending");
				Lending expiredLending = Lending.fromResource(lendingResource, model);
				expiredLendings.add(expiredLending);
			}
		} catch (Exception e) {
			LOG.error("error while fetching expired lendings", e);
			throw new WebApplicationException(Response.serverError().entity(e.getLocalizedMessage()).build());
		}
		
		return Response.ok(fm.render("lendings.ftl", "lendings", expiredLendings)).build();
	}
	
}
